@@ -26,6 +26,7 @@
 #include "xkbcommon/xkbcommon.h"
 
 #include "src/utils.h"
+#include "src/keymap-formats.h"
 #include "tools-common.h"
 
 struct keyboard {
@@ -43,7 +44,7 @@ static bool report_state_changes;
 static bool with_compose;
 static enum xkb_consumed_mode consumed_mode = XKB_CONSUMED_MODE_XKB;
 
-print_state_fields_mask_t print_fields = PRINT_ALL_FIELDS;
+enum print_state_options print_options = DEFAULT_PRINT_OPTIONS;
 
 #define DEFAULT_INCLUDE_PATH_PLACEHOLDER "__defaults__"
 #define NLONGS(n) (((n) + LONG_BIT - 1) / LONG_BIT)
@@ -257,12 +258,11 @@ process_event(struct keyboard *kbd, uint16_t type, uint16_t code, int32_t value)
         xkb_compose_state_feed(kbd->compose_state, keysym);
     }
 
-    if (value != KEY_STATE_RELEASE) {
-        tools_print_keycode_state(
-            NULL, kbd->state, kbd->compose_state, keycode,
-            consumed_mode, print_fields
-        );
-    }
+    tools_print_keycode_state(
+        NULL, kbd->state, kbd->compose_state, keycode,
+        (value == KEY_STATE_RELEASE) ? XKB_KEY_UP : XKB_KEY_DOWN,
+        consumed_mode, print_options
+    );
 
     if (with_compose) {
         status = xkb_compose_state_get_status(kbd->compose_state);
@@ -270,13 +270,13 @@ process_event(struct keyboard *kbd, uint16_t type, uint16_t code, int32_t value)
             xkb_compose_state_reset(kbd->compose_state);
     }
 
-    if (value == KEY_STATE_RELEASE)
-        changed = xkb_state_update_key(kbd->state, keycode, XKB_KEY_UP);
-    else
-        changed = xkb_state_update_key(kbd->state, keycode, XKB_KEY_DOWN);
+    changed = xkb_state_update_key(kbd->state, keycode,
+                                   (value == KEY_STATE_RELEASE
+                                        ? XKB_KEY_UP
+                                        : XKB_KEY_DOWN));
 
     if (report_state_changes)
-        tools_print_state_changes(changed);
+        tools_print_state_changes(NULL, kbd->state, changed, print_options);
 }
 
 static int
@@ -355,7 +355,7 @@ sigintr_handler(int signum)
 static void
 usage(FILE *fp, char *progname)
 {
-        fprintf(fp, "Usage: %s [--include=<path>] [--include-defaults] "
+        fprintf(fp, "Usage: %s [--include=<path>] [--include-defaults] [--format=<format>]"
                 "[--rules=<rules>] [--model=<model>] [--layout=<layout>] "
                 "[--variant=<variant>] [--options=<options>] "
                 "[--enable-environment-names]\n",
@@ -363,8 +363,11 @@ usage(FILE *fp, char *progname)
         fprintf(fp, "   or: %s --keymap <path to keymap file>\n",
                 progname);
         fprintf(fp, "For both:\n"
+                        "          --format <FORMAT> (use keymap format FORMAT)\n"
                         "          --verbose (enable verbose debugging output)\n"
-                        "          --short (do not print layout nor Unicode keysym translation)\n"
+                        "          -1, --uniline (enable uniline event output)\n"
+                        "          --multiline (enable uniline event output)\n"
+                        "          --short (shorter event output)\n"
                         "          --report-state-changes (report changes to the state)\n"
                         "          --enable-compose (enable Compose)\n"
                         "          --consumed-mode={xkb|gtk} (select the consumed modifiers mode, default: xkb)\n"
@@ -385,6 +388,7 @@ main(int argc, char *argv[])
     const char *includes[64];
     size_t num_includes = 0;
     bool use_env_names = false;
+    enum xkb_keymap_format keymap_format = DEFAULT_INPUT_KEYMAP_FORMAT;
     const char *rules = NULL;
     const char *model = NULL;
     const char *layout = NULL;
@@ -395,9 +399,12 @@ main(int argc, char *argv[])
     struct sigaction act;
     enum options {
         OPT_VERBOSE,
+        OPT_UNILINE,
+        OPT_MULTILINE,
         OPT_INCLUDE,
         OPT_INCLUDE_DEFAULTS,
         OPT_ENABLE_ENV_NAMES,
+        OPT_KEYMAP_FORMAT,
         OPT_RULES,
         OPT_MODEL,
         OPT_LAYOUT,
@@ -413,9 +420,12 @@ main(int argc, char *argv[])
     static struct option opts[] = {
         {"help",                 no_argument,            0, 'h'},
         {"verbose",              no_argument,            0, OPT_VERBOSE},
+        {"uniline",              no_argument,            0, OPT_UNILINE},
+        {"multiline",            no_argument,            0, OPT_MULTILINE},
         {"include",              required_argument,      0, OPT_INCLUDE},
         {"include-defaults",     no_argument,            0, OPT_INCLUDE_DEFAULTS},
         {"enable-environment-names", no_argument,        0, OPT_ENABLE_ENV_NAMES},
+        {"format",               required_argument,      0, OPT_KEYMAP_FORMAT},
         {"rules",                required_argument,      0, OPT_RULES},
         {"model",                required_argument,      0, OPT_MODEL},
         {"layout",               required_argument,      0, OPT_LAYOUT},
@@ -435,7 +445,7 @@ main(int argc, char *argv[])
     bool has_rmlvo_options = false;
     while (1) {
         int option_index = 0;
-        int opt = getopt_long(argc, argv, "h", opts, &option_index);
+        int opt = getopt_long(argc, argv, "*1h", opts, &option_index);
         if (opt == -1)
             break;
 
@@ -443,6 +453,14 @@ main(int argc, char *argv[])
         case OPT_VERBOSE:
             verbose = true;
             break;
+        case '1':
+        case OPT_UNILINE:
+            print_options |= PRINT_UNILINE;
+            break;
+        case '*':
+        case OPT_MULTILINE:
+            print_options &= ~PRINT_UNILINE;
+            break;
         case OPT_INCLUDE:
             if (num_includes >= ARRAY_SIZE(includes))
                 goto too_many_includes;
@@ -456,6 +474,14 @@ main(int argc, char *argv[])
         case OPT_ENABLE_ENV_NAMES:
             use_env_names = true;
             break;
+        case OPT_KEYMAP_FORMAT:
+            keymap_format = xkb_keymap_parse_format(optarg);
+            if (!keymap_format) {
+                fprintf(stderr, "ERROR: invalid --format \"%s\"\n", optarg);
+                usage(stderr, argv[0]);
+                return EXIT_INVALID_USAGE;
+            }
+            break;
         case OPT_RULES:
             if (keymap_path)
                 goto input_format_error;
@@ -501,7 +527,7 @@ main(int argc, char *argv[])
             with_compose = true;
             break;
         case OPT_SHORT:
-            print_fields &= ~PRINT_VERBOSE_FIELDS;
+            print_options &= ~PRINT_VERBOSE;
             break;
         case OPT_CONSUMED_MODE:
             if (strcmp(optarg, "gtk") == 0) {
@@ -541,6 +567,10 @@ too_much_arguments:
         }
     }
 
+    if (!(print_options & PRINT_VERBOSE) && (print_options & PRINT_UNILINE)) {
+        print_options &= ~PRINT_VERBOSE_ONE_LINE_FIELDS;
+    }
+
     enum xkb_context_flags ctx_flags = XKB_CONTEXT_NO_DEFAULT_INCLUDES;
     if (!use_env_names)
         ctx_flags |= XKB_CONTEXT_NO_ENVIRONMENT_NAMES;
@@ -551,10 +581,8 @@ too_much_arguments:
         goto out;
     }
 
-    if (verbose) {
-        xkb_context_set_log_level(ctx, XKB_LOG_LEVEL_DEBUG);
-        xkb_context_set_log_verbosity(ctx, 10);
-    }
+    if (verbose)
+        tools_enable_verbose_logging(ctx);
 
     if (num_includes == 0)
         includes[num_includes++] = DEFAULT_INCLUDE_PATH_PLACEHOLDER;
@@ -574,8 +602,7 @@ too_much_arguments:
                     keymap_path, strerror(errno));
             goto out;
         }
-        keymap = xkb_keymap_new_from_file(ctx, file,
-                                          XKB_KEYMAP_FORMAT_TEXT_V1,
+        keymap = xkb_keymap_new_from_file(ctx, file, keymap_format,
                                           XKB_KEYMAP_COMPILE_NO_FLAGS);
         fclose(file);
     }
@@ -589,9 +616,11 @@ too_much_arguments:
         };
 
         if (!rules && !model && !layout && !variant && !options)
-            keymap = xkb_keymap_new_from_names(ctx, NULL, 0);
+            keymap = xkb_keymap_new_from_names2(ctx, NULL, keymap_format,
+                                                XKB_KEYMAP_COMPILE_NO_FLAGS);
         else
-            keymap = xkb_keymap_new_from_names(ctx, &rmlvo, 0);
+            keymap = xkb_keymap_new_from_names2(ctx, &rmlvo, keymap_format,
+                                                XKB_KEYMAP_COMPILE_NO_FLAGS);
 
         if (!keymap) {
             fprintf(stderr,
